import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/** This is a subclass of Actor that is designed to work with the SlidingBoard world.
 * <p>The World class supports cells, but it doesn't support smoothly animating between
 * two cells. The SlidingBoard and SlidingBoardActor support sliding from one cell to another
 * smoothly.
 * <p>Also this class implements animated costumes (similar to an animated gif), so every
 * few cycles an object can change its appearance.
 * 
 */
public class SlidingActor extends Actor
{
    
    String[] costumeFilenames;
    
    int costumeCounter = 0;
    int changeCostumeCountdown = -1;
    int costumeCycleMax = 14;
    
    double ax = 0;
    double bx = 0;
    double ay = 0;
    double by = 0;
    int slidingCounter = 100000;
    int maxSlidingCycleCount = 0;
    
    public int getBoardTileSize() {
        return getImage().getWidth();
    }
    
    /** Begin sliding to a new board position.
     * @param boardX the board's x-value (starting at zero)
     * @param boardY the board's y-value (starting at zero)
     * @param cycleCount the number of cycles this slide should take.
     */
    public void slideToBoardPosition(double boardX,double boardY,int cycleCount) {
        int s = getBoardTileSize();
        double newX = boardX*s + s/2;
        double newY = boardY*s + s/2;
        ax = (newX - getX());
        bx = getX();
        ay = (newY - getY());
        by = getY();
        slidingCounter = 0;
        maxSlidingCycleCount = cycleCount;
    }
    
    /** Return true if this actor is centered in the board. */
    public boolean isCenteredInTile() {
        int boardX = getBoardX();
        int boardY = getBoardY();
        int s = getBoardTileSize();
        
        int x = getX();
        int y = getY();
       
        x = x - s/2;
        y = y - s/2;
        int column = x/s;
        int row = y/s;
        
        if(column*s==x && row*s==y) {
            return true;
        }
        return false;
    }
    
    /** This updates the position of this actor in the grid. Please override
     * the "recalculateDirection" method to change how this piece moves.
     */
    public void act() {
        int pixels = 0;
        slidingCounter++;
        if(slidingCounter<maxSlidingCycleCount) {
            double t = ((double)slidingCounter)/((double)maxSlidingCycleCount);
            double newX = ax*t+bx;
            double newY = ay*t+by;
            setLocation( (int)(newX), (int)(newY) );
        } else if(slidingCounter==maxSlidingCycleCount) {
            setLocation( (int)(ax + bx), (int)(ay + by) );
            recalculatePosition();
        } else {
            recalculatePosition();
        }
        updateCostumeCounter();
    }
    
    /** Returns the row this actor is closest to. */
    public int getBoardY() {
        return getY()/getBoardTileSize();
    }
    
    /** Returns the column this actor is closest to. */
    public int getBoardX() {
        return getX()/getBoardTileSize();
    }
    
    /** This method is called whenever an animation is complete. When this method
     * is called: it's appropriate for this Actor to consider calling slideToBoardPosition(..)
     */
    public void recalculatePosition() {
    }
    
    /** Set the costumes this Actor should cycle through. 
     * @param filenames an array of all the image filenames to cycle through.
     * @param cycleCount the number of cycles to wait before changing the costume.
     */
    public void setCostumes(String[] filenames,int cycleCount) {
        costumeFilenames = filenames;
        updateCostumeCounter();
    }
    
    /** This is increases the costume counter, and if necessary it will change the costume.
     * This is automatically called in the act() method, so you don't need to call this
     * unless you've overriden the act() method.
     */
    public void updateCostumeCounter() 
    {
        if(costumeFilenames==null) return;
        
        if(changeCostumeCountdown<0) {
            showNextCostume();
            changeCostumeCountdown = costumeCycleMax;
        }
        changeCostumeCountdown--;
    }
    
    /** This actually shows the next costume. */
    private void showNextCostume() {
        setImage(costumeFilenames[costumeCounter]);
        costumeCounter = costumeCounter + 1;
        if(costumeCounter>=costumeFilenames.length) {
            costumeCounter = 0;
        }
    }
}
